import ConfigParser

PURPOSES = {
    'automirror': "Automatically update a remote working tree upon commit",
    'automv': "Automatically detect renames/moves in the working tree",
    'bazimport': "Import an Arch or Baz archive",
    'bisect': "Find the revision introducing a bug using a binary search",
    'bookmarks': "Bookmarking system for often used branch locations/URLs",
    'branchfeed': "Create ATOM feeds from commit/push/pull actions",
    'builddeb': "Build Debian packages from Bazaar branches",
    'builder': "Construct a branch from a recipe",
    'bzrtools': "Collection of various useful commands",
    'checkeol': "Pre-commit hook to check files for correct line-endings",
    'cia': "Submit commits to CIA (open source VCS informant)",
    #'completion': None,
    'cvsps_import': "Import CVS branches using cvsps to extract patchsets",
    'dbus': "D-Bus integration and optional local-LAN commit broadcasting",
    'diffstat': "Show diff statistics for a revision or patch",
    'difftools': "Show differences using external GUI tools",
    'dns_cache': "Cache DNS requests, speeding up URL open requests",
    'docdiff': "Show differences for Office (e.g. Word, Excel) documents",
    'email': "Send email upon commit",
    #'explorer': None,
    'externals': "Support external branches ala Subversion",
    'extmerge': "Merge using external GUI tools",
    'fastimport': "Import/export of history via fast-import format",
    'git': "Transparent interoperability with Git branches",
    'gtk': "GTK dialogs for many Bazaar commands",
    'hg': "Transparent interoperability with Mercurial branches",
    'historycache': "Faster lookup of dotted revision numbers and history graphs",
    #'interactive': None,
    'keywords': "Keyword templating (e.g. Date, Authors, etc.)",
    'launchpad': "Integration with Launchpad.net (project hosting service)",
    'loggerhead': "Web interface for browsing branches",
    'loom': "Manage a loom of patches",
    'merge_into': "Merge another project into a subdirectory of this project",
    #'bzr_notification': None,
    'pager': "Pipe commands producing long output to a pager (e.g. less)",
    'ping': "Ping a smart server and report the response",
    'pipeline': "Manage a series of branches as a pipeline",
    'pqm': "Submit changes to a Patch Queue Manager",
    'push_and_update': "Update a remote working tree upon push",
    'qbzr': "Qt dialogs for many Bazaar commands",
    'rebase': "Rewrite history by replaying patches",
    'repodetails': "Analyze repository storage and report results",
    'repo_push': "Push a repository and all its branches to another location",
    'scmproj': "Organize branches into a unified project",
    'search': "Full text index and search of a repository",
    'service': "Keep bzrlib loaded in memory to decrease command startup time",
    'stats': "Generate history statistics (e.g. committer contributions)",
    'svn': "Transparent interoperability with Subversion branches",
    #'bzr_tcsh_completion': None,
    'testrunner': "Run a command before allowing a commit",
    'text_checker': "Avoid trailing whitespace and other undesired text",
    'upload': "Upload a working tree with incremental updates",
    'vimdiff': "Show differences using vimdiff",
    'webdav': "Support writing to WebDAV branches",
    'win32symlinks': "Fake symlinks in cygwin format on win32 native",
    'x_bit': "Control executable bit of particular files",
    'xmloutput': "XML interface for external tool integration",
    }

AUTHORS = {
    "AaronBentley": "abentley",
    "AlexanderBelchenko": "bialix",
    "AlexanderBelchenko', 'MariusKruger": "bialix",
    "AliSabil": "asabil",
    "BarryWarsaw": "barry",
    "BrianDeAlwis": "slyguy",
    "Carl Friedrich Bolz": "cfbolz",
    "ChristopherArmstrong": "radix",
    "EduardoPadoan": "eopadoan",
    r"ErikB\xc3\xa5gfors": "zindar",
    "FriedrichWeber": "fredreichbier",
    "GoffredoBaroncelli": "kreijack",
    "GuillermoGonzalez": "verterok",
    "GustavoCarneiro": "gjc",
    "IanClatworthy": "ian-clatworthy",
    "JamesHenstridge": "jamesh",
    "JamesWestby": "james-w",
    "JasonSmith": "jassmith",
    "JelmerVernooij": "jelmer",
    "JohanDahlin": "jdahlin",
    "JohnArbashMeinel": "jameinel",
    "KeirMierle": "mierle",
    r"Luk\xc3\xa1\xc5\xa1 Lalinsk\xc3\xbd": "luks",
    "LukasLalinsky": "luks",
    "Marien Zwart": "marienz",
    "MariusKruger": "amanica",
    "MartinPool": "mnp",
    "MichaelEllerman": "michael-ellerman",
    "MichaelHudson and MartinAlbisetti": "mwhudson",
    "NeilMartinsenBurrell": "nmb",
    "ParthMalwankar": "parthm",
    "PascalBach": "pascal-bach",
    "PaulHinze": "phinze",
    "Pileofrogs": "dmartin-sccd",
    "RobertCollins": "lifeless",
    "RobertCollins', 'JelmerVernooij": "jelmer",
    "RobertWidhopf": "hack-robf",
    "StephenWard": "sward-dev",
    "TimPowell": "theuiguy",
    "VincentLadeuil": "vila",
    "VincentLadeuil', 'MartinAlbisetti": "vila",
    "WouterBolsterlee": "uws",
    "[[Russ Brown]]": "pickscape",
    "[[gagern|Martin von Gagern]]": "gagern",
    "[[https://launchpad.net/~bzr-explorer-dev|Bazaar Explorer team]]": "bzr-explorer-dev",
    "[[https://launchpad.net/~bzr-gtk|Bazaar-GTK team]]": "bzr-gtk",
    "[[kenhys| HAYASHI Kentaro]]": "kenhys",
    }

def find_lp_owner(s):
    s = s[2:-2]
    if s:
        return AUTHORS[s] or s
    return None

config = ConfigParser.ConfigParser()
config.read(['plugins-v1.ini'])
for section in sorted(config.sections()):
    # Get the data and clean it up
    data = dict(config.items(section))
    desc = data['description']
    del data['description']
    plugin_name = section.split()[1]
    purpose = PURPOSES.get(plugin_name, desc)
    if purpose != desc:
        data['wiki.description'] = desc
    owner = find_lp_owner(data['owners'])
    del data['owners']
    if owner:
        data['owner'] = owner

    # Format it
    print '[%s]' % (section,)
    print 'purpose = %s' % (purpose,)
    for name, value in sorted(data.items()):
        print "%s = %s" % (name, value)
    print
