categories = (
    ('Web', [
        'launchpad',
        'loggerhead',
        'webdav',
        'webserve',
        'xmloutput',
        ]),
    ('Desktop', [
        'encdiff',
        'explorer',
        'extmerge',
        'commit-selector',
        'difftools',
        'gtk',
        'interactive',
        'proe',
        'qbzr',
        'vimdiff',
        ]),
    ('Publishing', [
        'automirror',
        'keywords',
        'nicehtml',
        'push-and-update',
        'repo-push',
        'upload',
        ]),
    ('Change notification', [
        'atomlog',
        'branchfeed',
        'cia',
        'email',
        'publish-bot',
        'shell-hooks',
        ]),
    ('Change control', [
        'checkeol',
        'pqm',
        'testrunner',
        'text-checker',
        ]),
    ('History exploration', [
        'bisect',
        'countlog',
        'diffstat',
        'file-revno',
        'gnulog',
        'htmllog',
        'grep',
        'hgrep',
        'lastlog',
        'search',
        'stats',
        ]),
    ('Change ordering', [
        'loom',
        'pipeline',
        'rebase',
        'remove-revisions',
        ]),
    ('Metaprojects', [
        'builder',
        'externals',
        'scmproj',
        'merge-into',
        ]),
    ('Productivity', [
        'automv',
        'bookmarks',
        'bzrtools',
        'extcommand',
        'ezbzr',
        'lightweight-branch',
        'recursive-upgrade',
        'repoalias',
        'rxrename',
        'touch',
        'undelete',
        'update-mirrors',
        ]),
    ('Foreign branches', [
        'cvsserve',
        'git',
        'hg',
        'svn',
        ]),
    ('Migration', [
        'bazimport',
        'cvsps-import',
        'fastimport',
        ]),
    ('Performance tuning', [
        'dns-cache',
        'gzipped-bundle',
        'historycache',
        'network-manager',
        'pack-indices',
        'quickbranch',
        'service',
        ]),
    ('Diagnostics', [
        'depend',
        'ping',
        'repodetails',
        ]),
    ('Testing', [
        'benchdisplay',
        'usertest',
        ]),
    ('Linux/UNIX specific', [
        'bash-completion',
        'builddeb',
        'dbus',
        'notification',
        'lessdiff',
        'lesslog',
        'pager',
        'tcsh-completion',
        ]),
    ('Windows specific', [
        'docdiff',
        'win32symlinks',
        'x-bit',
        ]),
    ('OS X specific', [
        'mail-app',
        ]),
    ('Miscellaneous', [
        'setuptoolsbzr',
        ]),
    ('Experimental', [
        'colocated',
        ]),
    )


def wiki_to_cfg(wiki_file, cfg_file):
    f = open(wiki_file, 'r')
    lines = f.readlines()
    f.close()
    entries, people = parse_wiki_lines(lines)
    cfg = open(cfg_file, 'w')
    #format_people(people, cfg)
    format_entries(entries, cfg)
    cfg.close()
    return entries


def parse_wiki_lines(lines):
    # Get the interesting lines
    data = []
    for line in lines:
        if not line.startswith('||'):
            continue
        # Skip header rows
        if line.find("'''Platform'''") >= 0:
            continue
        if line.find("'''Mac'''") >= 0:
            continue
        if line.find("'''Home'''") >= 0:
            continue
        data.append(line)

    # Convert them to entries
    people = set()
    entries = {}
    for i in range(0, len(data), 2):
        name, details = parse_entry(data[i], data[i + 1])
        if details is None:
            continue
        entries[name] =  details
        people.update(details['owners'])
    return entries, people


def format_people(people, outf):
    outf.write("[lookup owner]\n")
    for name in sorted(people):
        outf.write("%s\n" % (name,))


def format_entries(entries, outf):
    for name in sorted(entries):
        details = entries[name]
        lines = ["%s = %s" % (k, v) for k, v in details.items()]
        details_str = "\n".join(lines)
        outf.write("[plugin %s]\n%s\n\n" % (name, details_str))


def parse_entry(row1, row2):
    """Extract the information for a plugin from a pair of rows.
    
    The data is expected to be in wiki-style table rows. The first row
    contains name, part-of, owner, and 3 field for platform compatibility.
    The second row contains a description and branch URLs.

    :param row1: text for the first row
    :param row2: text for the second row
    :return: a dictionary of attributes
    """
    info = {}

    # Parse the first row
    parts = row1[2:].split('||')
    if parts[1].strip() == 'BzrTools':
        return None, None
    name, home = _parse_name_and_home(parts[0])
    info['owners'] = _parse_owners(parts[2])
    _parse_platform(parts[3], info, 'linux')
    _parse_platform(parts[4], info, 'windows')
    _parse_platform(parts[5], info, 'mac')

    # Parse the second row
    parts = row2[2:].split('||')
    info['description'] = _parse_description(parts[1])
    info.update(_parse_urls(parts[5], home))
    return name, info


def _parse_name_and_home(s):
    s = s.strip()
    if s.startswith("'''"):
        s = s[3:]
    if s.endswith("'''"):
        s = s[:-3]
    if s.startswith('[['):
        link, name = _parse_wiki_url(s)
        if name is None:
            print "failed to parse row %s" % (row1,)
            return None, link
    else:
        name = s
        link = None
    name = name.replace('_', '-')
    if name.startswith('bzr-'):
        name = name[4:]
    if name.startswith('bzrcvs'):
        name = name[3:]
    elif name == 'foreach':
        # hack in the containing plugin
        name = 'ezbzr'
    return name, link


def _parse_owners(s):
    s = s.strip()
    if s.find(',') >= 0:
        return [owner.strip() for owner in s.split(',')]
    else:
        return [s]


def _parse_platform(s, info, name):
    s = s.strip()
    if s == '(./)':
        info['platform.%s' % name] = True
    elif s in ['X', 'x']:
        info['platform.%s' % name] = False


def _parse_description(s):
    return _strip_formatting(s.strip())


def _strip_formatting(s):
    formatting = '<style="text-align: center;">'
    if s.startswith(formatting):
        s = s[len(formatting):]
    return s.strip()


def _parse_urls(s, default_home):
    # return a dictionary of interesting urls
    s = _strip_formatting(s.strip())
    second_start = s[2:].find('[[')
    if second_start >= 0:
        if s.find(' or ') >= 0:
            parts = s.split(' or ')
        elif s.find('<<BR>>') >= 0:
            parts = s.split(' <<BR>> ')
        else:
            # TODO: handle > 2 URLs
            #parts = [s[:second_start + 2], s[second_start + 2:]]
            parts = ['[[' + p for p in s[2:].split('[[')]
        parts = [p.strip() for p in parts]
    else:
        parts = [s]
    result = {}  # {'link-text': s}
    if default_home:
        result['link.home'] = default_home
    for part in parts:
        link, text = _parse_wiki_url(part)
        if text in ['URL to branch', 'Branch', 'Stable branch']:
            result['link.branch'] = link
        elif text in ['Download']:
            result['link.download'] = link
        elif text in ['Home page', 'URL to Project', 'Project', 'Countlog', None]:
            result['link.home'] = link
        elif text in ['URL to development branch', 'Dev branch']:
            result['link.development'] = link
        elif text in ['Browse', 'Source view']:
            result['link.browse'] = link
        else:
            print "Cannot classify URL %s" % (part,)
            print "parts = %s" % (parts,)
    return result


def _parse_wiki_url(s):
    # Parse url text into a url and description.
    # Return None, None if not a URL.
    if s.startswith('[[') and s.endswith(']]'):
        s = s[2:-2]
    elif not s.startswith('http'):
        print "cannot parse url '%s'" % (s,)
        return None, None
    if s == 'QBzr':
        return 'http://bazaar-vcs.org/QBzr', 'qbzr'
    elif s.find('|') >= 0:
        link, name = s.split('|', 1)
        return link.strip(), name.strip()
    return s, None


def _parse_wiki_url(s):
    # Parse url text into a url and description.
    # Return None, None if not a URL.
    if s.startswith('[[') and s.endswith(']]'):
        s = s[2:-2]
    elif not s.startswith('http'):
        print "cannot parse url '%s'" % (s,)
        return None, None
    if s == 'QBzr':
        return 'http://bazaar-vcs.org/QBzr', 'qbzr'
    elif s.find('|') >= 0:
        link, name = s.split('|', 1)
        return link.strip(), name.strip()
    return s, None


_WIKI_HEADER_ROWS = \
        """||<bgcolor="#cccccc"> '''Name''' ||<bgcolor="#cccccc" style="text-align: left;"> '''Description''' ||<bgcolor="#cccccc" style="text-align: center;"> '''URLs''' ||<bgcolor="#cccccc" style="text-align: center;"> '''Owner''' ||<bgcolor="#cccccc"> '''Platform''' ||
"""
#|||||||||||| <bgcolor="#cccccc" style="text-align: left;"> '''Description''' ||


def format_wiki_page(entries, categories, outf):
    unknown = set(entries.keys())
    for category, plugins in categories:
        outf.write("\n\n== %s ==\n\n" % (category,))
        outf.write(_WIKI_HEADER_ROWS)
        for plugin in plugins:
            if plugin in entries:
                format_wiki_entry(plugin, entries[plugin], outf)
                unknown.remove(plugin)
    rest = sorted(unknown)
    if rest:
        outf.write("\n\n= %s =\n\n" % ('Unclassified',))
        outf.write(_WIKI_HEADER_ROWS)
        for plugin in rest:
            format_wiki_entry(plugin, entries[plugin], outf)


def format_wiki_entry(plugin, details, outf):
    if 'home' in details:
        name = '[[%s|%s]]' % (details['home'], plugin)
    else:
        name = plugin
    name = "'''%s'''" % (name,)
    part_of = details.get('part_of', '')
    owners = ", ".join(details['owners'])
    linux = format_wiki_platform(details, 'linux')
    windows = format_wiki_platform(details, 'windows')
    mac = format_wiki_platform(details, 'mac')
    platform = " ".join([linux, windows, mac])
    desc = details.get('description', '')
    links = format_wiki_links(details)
    # output row 1
    lalign = '<style="text-align: left;">'
    align = '<style="text-align: center;">'
    outf.write('|| %s ||%s%s ||%s%s ||%s%s || %s ||\n' %
        (name, lalign, desc, align, links, align, owners, platform))
    # output row 2
    #outf.write('||||||||||||%s%s ||\n' % (align, desc))


def format_wiki_platform(details, name):
    flag = details.get('platform.%s' % name)
    if flag:
        #return '(./)'
        return '{{attachment:%s.png}}' % name
    elif flag is None:
        #return '?'
        return ''
    else:
        #return 'X'
        return '{{attachment:%s-no.png}}' % name


def format_wiki_links(details):
    links = []
    for link in ['home', 'download', 'branch', 'browse', 'development']:
        key = 'link.%s' % link
        if key in details:
            title = link[0].upper() + link[1:]
            links.append('[[%s|%s]]' % (details[key], title))
    return ', '.join(links)


entries = wiki_to_cfg('wiki-data.txt', 'plugins-generated.cfg')
import sys
format_wiki_page(entries, categories, sys.stdout)
